@@ -58,9 +58,9 @@ let add_property_field com c =
 			let cf = mk_field n com.basic.tstring p null_pos in
 			PMap.add n cf fields,((n,null_pos,NoQuotes),Texpr.Builder.make_string com.basic v p) :: values
 		) (PMap.empty,[]) props in
-		let t = mk_anon fields in
+		let t = mk_anon ~fields (ref Closed) in
 		let e = mk (TObjectDecl values) t p in
-		let cf = mk_field "__properties__" t p null_pos in
+		let cf = mk_field ~static:true "__properties__" t p null_pos in
 		cf.cf_expr <- Some e;
 		c.cl_statics <- PMap.add cf.cf_name cf c.cl_statics;
 		c.cl_ordered_statics <- cf :: c.cl_ordered_statics
@@ -94,7 +94,7 @@ let update_cache_dependencies t =
 		| TAnon an ->
 			PMap.iter (fun _ cf -> check_field m cf) an.a_fields
 		| TMono r ->
-			(match !r with
+			(match r.tm_type with
 			| Some t -> check_t m t
 			| _ -> ())
 		| TLazy f ->
@@ -154,7 +154,7 @@ let fix_override com c f fd =
 	let f2 = (try Some (find_field com c f) with Not_found -> None) in
 	match f2,fd with
 		| Some (f2), Some(fd) ->
-			let targs, tret = (match follow f2.cf_type with TFun (args,ret) -> args, ret | _ -> assert false) in
+			let targs, tret = (match follow f2.cf_type with TFun (args,ret) -> args, ret | _ -> die "" __LOC__) in
 			let changed_args = ref [] in
 			let prefix = "_tmp_" in
 			let nargs = List.map2 (fun ((v,ct) as cur) (_,_,t2) ->
@@ -189,14 +189,12 @@ let fix_override com c f fd =
 						{ e with eexpr = TBlock (el_v @ el) }
 				);
 			} in
-			(* as3 does not allow wider visibility, so the base method has to be made public *)
-			if Common.defined com Define.As3 && has_class_field_flag f CfPublic then add_class_field_flag f2 CfPublic;
 			let targs = List.map (fun(v,c) -> (v.v_name, Option.is_some c, v.v_type)) nargs in
-			let fde = (match f.cf_expr with None -> assert false | Some e -> e) in
+			let fde = (match f.cf_expr with None -> die "" __LOC__ | Some e -> e) in
 			f.cf_expr <- Some { fde with eexpr = TFunction fd2 };
 			f.cf_type <- TFun(targs,tret);
 		| Some(f2), None when c.cl_interface ->
-			let targs, tret = (match follow f2.cf_type with TFun (args,ret) -> args, ret | _ -> assert false) in
+			let targs, tret = (match follow f2.cf_type with TFun (args,ret) -> args, ret | _ -> die "" __LOC__) in
 			f.cf_type <- TFun(targs,tret)
 		| _ ->
 			()
@@ -247,7 +245,7 @@ let fix_abstract_inheritance com t =
 let rec is_volatile t =
 	match t with
 	| TMono r ->
-		(match !r with
+		(match r.tm_type with
 		| Some t -> is_volatile t
 		| _ -> false)
 	| TLazy f ->
@@ -422,7 +420,7 @@ module Dump = struct
 			| None -> platform_name_macro com
 			| Some s -> s
 		in
-		let dump_dependencies_path = [dump_path com;target_name;".dependencies"] in
+		let dump_dependencies_path = [dump_path com;target_name;"dependencies"] in
 		let buf,close = create_dumpfile [] dump_dependencies_path in
 		let print fmt = Printf.kprintf (fun s -> Buffer.add_string buf s) fmt in
 		let dep = Hashtbl.create 0 in
@@ -435,7 +433,7 @@ module Dump = struct
 			) m.m_extra.m_deps;
 		) com.Common.modules;
 		close();
-		let dump_dependants_path = [dump_path com;target_name;".dependants"] in
+		let dump_dependants_path = [dump_path com;target_name;"dependants"] in
 		let buf,close = create_dumpfile [] dump_dependants_path in
 		let print fmt = Printf.kprintf (fun s -> Buffer.add_string buf s) fmt in
 		Hashtbl.iter (fun n ml ->
@@ -454,21 +452,21 @@ end
 let default_cast ?(vtmp="$t") com e texpr t p =
 	let api = com.basic in
 	let mk_texpr = function
-		| TClassDecl c -> TAnon { a_fields = PMap.empty; a_status = ref (Statics c) }
-		| TEnumDecl e -> TAnon { a_fields = PMap.empty; a_status = ref (EnumStatics e) }
-		| TAbstractDecl a -> TAnon { a_fields = PMap.empty; a_status = ref (AbstractStatics a) }
-		| TTypeDecl _ -> assert false
+		| TClassDecl c -> mk_anon (ref (Statics c))
+		| TEnumDecl e -> mk_anon (ref (EnumStatics e))
+		| TAbstractDecl a -> mk_anon (ref (AbstractStatics a))
+		| TTypeDecl _ -> die "" __LOC__
 	in
 	let vtmp = alloc_var VGenerated vtmp e.etype e.epos in
 	let var = mk (TVar (vtmp,Some e)) api.tvoid p in
 	let vexpr = mk (TLocal vtmp) e.etype p in
 	let texpr = mk (TTypeExpr texpr) (mk_texpr texpr) p in
-	let std = (try List.find (fun t -> t_path t = ([],"Std")) com.types with Not_found -> assert false) in
+	let std = (try List.find (fun t -> t_path t = ([],"Std")) com.types with Not_found -> die "" __LOC__) in
 	let fis = (try
-			let c = (match std with TClassDecl c -> c | _ -> assert false) in
-			FStatic (c, PMap.find "is" c.cl_statics)
+			let c = (match std with TClassDecl c -> c | _ -> die "" __LOC__) in
+			FStatic (c, PMap.find "isOfType" c.cl_statics)
 		with Not_found ->
-			assert false
+			die "" __LOC__
 	) in
 	let std = mk (TTypeExpr std) (mk_texpr std) p in
 	let is = mk (TField (std,fis)) (tfun [t_dynamic;t_dynamic] api.tbool) p in
@@ -511,7 +509,7 @@ module UnificationCallback = struct
 		in
 		let check e = match e.eexpr with
 			| TBinop((OpAssign | OpAssignOp _),e1,e2) ->
-				assert false; (* this trigger #4347, to be fixed before enabling
+				die "" __LOC__; (* this trigger #4347, to be fixed before enabling
 				let e2 = f e2 e1.etype in
 				{e with eexpr = TBinop(op,e1,e2)} *)
 			| TVar(v,Some ev) ->
