@@ -279,7 +279,7 @@ let rec set_hxgeneric gen mds isfirst md =
 								Some true
 							end
 				end
-			| _ -> assert false
+			| _ -> Globals.die "" __LOC__
 	end
 
 let path_s = function
@@ -294,7 +294,7 @@ let set_hxgeneric gen md =
 					let md = match t with
 						| TInst(cl,_) -> TClassDecl cl
 						| TEnum(e,_) -> TEnumDecl e
-						| _ -> assert false
+						| _ -> Globals.die "" __LOC__
 					in
 					let ret = set_hxgeneric gen [] true md in
 					if ret = None then get (set_hxgeneric gen [] false md) else get ret)
@@ -341,7 +341,7 @@ let set_hxgeneric gen md =
 				"because it explicitly has the metadata @:nativeGeneric set"
 		in
 		gen.gcon.error (reason) pos;
-		assert false
+		Globals.die "" __LOC__
 
 let params_has_tparams params =
 	List.fold_left (fun acc t -> acc || has_type_params t) false params
@@ -506,7 +506,7 @@ struct
 				| TEnum (e,_) -> e.e_path
 				| TAbstract (a,_) -> a.a_path
 				| TMono _ | TDynamic _ -> ([], "Dynamic")
-				| _ -> assert false
+				| _ -> Globals.die "" __LOC__
 			in
 			List.map (fun (cf, t_cl, t_cf) ->
 				let t_cf = follow (gen.greal_type t_cf) in
@@ -518,7 +518,7 @@ struct
 						(try (Hashtbl.find gen.gtparam_cast (get_path t_cf)) this_field t_cf with Not_found ->
 							(* if not found tparam cast, it shouldn't be a valid hxgeneric *)
 							print_endline ("Could not find a gtparam_cast for " ^ (String.concat "." (fst (get_path t_cf)) ^ "." ^ (snd (get_path t_cf))));
-							assert false)
+							Globals.die "" __LOC__)
 						t_cf
 						pos
 				in
@@ -543,8 +543,8 @@ struct
 				| (TInst(cl1,[]) as v), (TInst(cl2,[]) as v2) ->
 					mk_typehandle_cond (v :: hd) (v2 :: hd2)
 				| _ ->
-					assert false)
-			| _ -> assert false
+					Globals.die "" __LOC__)
+			| _ -> Globals.die "" __LOC__
 		in
 		let fn = {
 			tf_args = [];
@@ -589,7 +589,7 @@ struct
 		let basic = gen.gcon.basic in
 		let cparams = List.map (fun (s,t) -> ("To_" ^ s, TInst (map_param (get_cl_t t), []))) cf.cf_params in
 		let me_type = TInst(iface,[]) in
-		let cfield = mk_class_field "__hx_cast" (TFun(["me",false,me_type], t_dynamic)) false iface.cl_pos (Method MethNormal) (cparams) in
+		let cfield = mk_class_field ~static:true "__hx_cast" (TFun(["me",false,me_type], t_dynamic)) false iface.cl_pos (Method MethNormal) (cparams) in
 		let params = List.map snd cparams in
 
 		let me = alloc_var "me" me_type in
@@ -712,7 +712,7 @@ struct
 							let cf_type = if is_override && not (Meta.has Meta.Overload cf.cf_meta) then
 								match find_first_declared_field gen cl cf.cf_name with
 									| Some(_,_,declared_t,_,_,_,_) -> declared_t
-									| _ -> assert false
+									| _ -> Globals.die "" __LOC__
 							else
 								cf.cf_type
 							in
