@@ -226,7 +226,15 @@ let resolve_pos file =
 		f
 
 let find_file file =
-	try Hashtbl.find all_files file with Not_found -> try resolve_pos file with Sys_error _ -> make_file file
+	try
+		Hashtbl.find all_files file
+	with Not_found ->
+		try
+			let f = resolve_pos file in
+			Hashtbl.add all_files file f;
+			f
+		with Sys_error _ ->
+			make_file file
 
 let find_pos p =
 	find_line p.pmin (find_file p.pfile)
@@ -256,6 +264,8 @@ let get_error_pos printer p =
 			Printf.sprintf "%s character%s" (printer p.pfile l1) s
 		end else
 			Printf.sprintf "%s lines %d-%d" (printer p.pfile l1) l1 l2
+;;
+Globals.get_error_pos_ref := get_error_pos
 
 let reset() = Buffer.reset buf
 let contents() = Buffer.contents buf
@@ -320,7 +330,7 @@ let rec skip_header lexbuf =
 	| 0xfeff -> skip_header lexbuf
 	| "#!", Star (Compl ('\n' | '\r')) -> skip_header lexbuf
 	| "" | eof -> ()
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 let rec token lexbuf =
 	match%sedlex lexbuf with
@@ -486,7 +496,7 @@ and comment lexbuf =
 	| "*/" -> lexeme_end lexbuf
 	| '*' -> store lexbuf; comment lexbuf
 	| Plus (Compl ('*' | '\n' | '\r')) -> store lexbuf; comment lexbuf
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 and string lexbuf =
 	match%sedlex lexbuf with
@@ -497,7 +507,7 @@ and string lexbuf =
 	| '\\' -> store lexbuf; string lexbuf
 	| '"' -> lexeme_end lexbuf
 	| Plus (Compl ('"' | '\\' | '\r' | '\n')) -> store lexbuf; string lexbuf
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 and string2 lexbuf =
 	match%sedlex lexbuf with
@@ -514,7 +524,7 @@ and string2 lexbuf =
 		(try code_string lexbuf 0 with Exit -> error Unclosed_code pmin);
 		string2 lexbuf;
 	| Plus (Compl ('\'' | '\\' | '\r' | '\n' | '$')) -> store lexbuf; string2 lexbuf
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 and code_string lexbuf open_braces =
 	match%sedlex lexbuf with
@@ -548,7 +558,7 @@ and code_string lexbuf open_braces =
 		code_string lexbuf open_braces
 	| "//", Star (Compl ('\n' | '\r')) -> store lexbuf; code_string lexbuf open_braces
 	| Plus (Compl ('/' | '"' | '\'' | '{' | '}' | '\n' | '\r')) -> store lexbuf; code_string lexbuf open_braces
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 and regexp lexbuf =
 	match%sedlex lexbuf with
@@ -563,7 +573,7 @@ and regexp lexbuf =
 	| '\\', Compl '\\' -> error (Invalid_character (Uchar.to_int (lexeme_char lexbuf 0))) (lexeme_end lexbuf - 1)
 	| '/' -> regexp_options lexbuf, lexeme_end lexbuf
 	| Plus (Compl ('\\' | '/' | '\r' | '\n')) -> store lexbuf; regexp lexbuf
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 and regexp_options lexbuf =
 	match%sedlex lexbuf with
@@ -572,7 +582,7 @@ and regexp_options lexbuf =
 		l ^ regexp_options lexbuf
 	| 'a'..'z' -> error Invalid_option (lexeme_start lexbuf)
 	| "" -> ""
-	| _ -> assert false
+	| _ -> die "" __LOC__
 
 and not_xml ctx depth in_open =
 	let lexbuf = ctx.lexbuf in
@@ -616,7 +626,7 @@ and not_xml ctx depth in_open =
 		store lexbuf;
 		not_xml ctx depth in_open
 	| _ ->
-		assert false
+		die "" __LOC__
 
 let rec sharp_token lexbuf =
 	match%sedlex lexbuf with
