@@ -107,7 +107,7 @@ let valid_redefinition ctx f1 t1 f2 t2 = (* child, parent *)
 				let msg = if !i = 0 then Invalid_return_type else Invalid_function_argument(!i,List.length args1) in
 				raise (Unify_error (Cannot_unify (t1,t2) :: msg :: l)))
 		| _ ->
-			assert false
+			die "" __LOC__
 		end
 	| _,(Var { v_write = AccNo | AccNever }) ->
 		(* write variance *)
@@ -149,14 +149,18 @@ let get_native_name meta =
 		error "String expected" mp
 
 let check_native_name_override ctx child base =
-	let error() =
-		display_error ctx ("Field " ^ child.cf_name ^ " has different @:native value than in superclass") child.cf_pos;
-		display_error ctx ("Base field is defined here") base.cf_pos
+	let error base_pos child_pos =
+		display_error ctx ("Field " ^ child.cf_name ^ " has different @:native value than in superclass") child_pos;
+		display_error ctx ("Base field is defined here") base_pos
 	in
 	try
-		let native_name = fst (get_native_name child.cf_meta) in
-		try if fst (get_native_name base.cf_meta) <> native_name then error()
-		with Not_found -> error()
+		let child_name, child_pos = get_native_name child.cf_meta in
+		try
+			let base_name, base_pos = get_native_name base.cf_meta in
+			if base_name <> child_name then
+				error base_pos child_pos
+		with Not_found ->
+			error base.cf_name_pos child_pos
 	with Not_found -> ()
 
 let check_overriding ctx c f =
@@ -165,7 +169,7 @@ let check_overriding ctx c f =
 		if List.memq f c.cl_overrides then display_error ctx ("Field " ^ f.cf_name ^ " is declared 'override' but doesn't override any field") f.cf_pos
 	| _ when c.cl_extern && Meta.has Meta.CsNative c.cl_meta -> () (* -net-lib specific: do not check overrides on extern CsNative classes *)
 	| Some (csup,params) ->
-		let p = f.cf_pos in
+		let p = f.cf_name_pos in
 		let i = f.cf_name in
 		let check_field f get_super_field is_overload = try
 			(if is_overload && not (Meta.has Meta.Overload f.cf_meta) then
@@ -197,7 +201,7 @@ let check_overriding ctx c f =
 			with
 				Unify_error l ->
 					display_error ctx ("Field " ^ i ^ " overrides parent class with different or incomplete type") p;
-					display_error ctx ("Base field is defined here") f2.cf_pos;
+					display_error ctx ("Base field is defined here") f2.cf_name_pos;
 					display_error ctx (error_msg (Unify l)) p;
 		with
 			Not_found ->
@@ -322,7 +326,7 @@ module Inheritance = struct
 	let check_extends ctx c t p = match follow t with
 		| TInst (csup,params) ->
 			if is_basic_class_path csup.cl_path && not (c.cl_extern && csup.cl_extern) then error "Cannot extend basic class" p;
-			if is_parent c csup then error "Recursive class" p;
+			if extends csup c then error "Recursive class" p;
 			begin match csup.cl_kind with
 				| KTypeParameter _ ->
 					if is_generic_parameter ctx csup then error "Extending generic type parameters is no longer allowed in Haxe 4" p;
@@ -332,7 +336,7 @@ module Inheritance = struct
 		| _ -> error "Should extend by using a class" p
 
 	let rec check_interface ctx c intf params =
-		let p = c.cl_pos in
+		let p = c.cl_name_pos in
 		let rec check_field i f =
 			(if ctx.com.config.pf_overload then
 				List.iter (function
@@ -351,12 +355,8 @@ module Inheritance = struct
 					else
 						t2, f2
 				in
-				if ctx.com.display.dms_collect_data then begin
-						let h = ctx.com.display_information in
-						h.interface_field_implementations <- (intf,f,c,Some f2) :: h.interface_field_implementations;
-				end;
 				ignore(follow f2.cf_type); (* force evaluation *)
-				let p = (match f2.cf_expr with None -> p | Some e -> e.epos) in
+				let p = f2.cf_name_pos in
 				let mkind = function
 					| MethNormal | MethInline -> 0
 					| MethDynamic -> 1
@@ -379,7 +379,7 @@ module Inheritance = struct
 				| Not_found when not c.cl_interface ->
 					let msg = if !is_overload then
 						let ctx = print_context() in
-						let args = match follow f.cf_type with | TFun(args,_) -> String.concat ", " (List.map (fun (n,o,t) -> (if o then "?" else "") ^ n ^ " : " ^ (s_type ctx t)) args) | _ -> assert false in
+						let args = match follow f.cf_type with | TFun(args,_) -> String.concat ", " (List.map (fun (n,o,t) -> (if o then "?" else "") ^ n ^ " : " ^ (s_type ctx t)) args) | _ -> die "" __LOC__ in
 						"No suitable overload for " ^ i ^ "( " ^ args ^ " ), as needed by " ^ s_type_path intf.cl_path ^ " was found"
 					else
 						("Field " ^ i ^ " needed by " ^ s_type_path intf.cl_path ^ " is missing")
@@ -436,7 +436,7 @@ module Inheritance = struct
 						List.find path_matches ctx.m.curmod.m_types
 					with Not_found ->
 						let t,pi = List.find (fun (lt,_) -> path_matches lt) ctx.m.module_types in
-						ImportHandling.mark_import_position ctx.com pi;
+						ImportHandling.mark_import_position ctx pi;
 						t
 					in
 					{ t with tpackage = fst (t_path lt) },p
@@ -475,7 +475,7 @@ module Inheritance = struct
 					c.cl_array_access <- Some t;
 					(fun () -> ())
 				| TInst (intf,params) ->
-					if is_parent c intf then error "Recursive class" p;
+					if extends intf c then error "Recursive class" p;
 					if c.cl_interface then error "Interfaces cannot implement another interface (use extends instead)" p;
 					if not intf.cl_interface then error "You can only implement an interface" p;
 					c.cl_implements <- (intf, params) :: c.cl_implements;
